@@ -4,6 +4,7 @@ from datetime import date
 from contextlib import contextmanager
 from pathlib import Path
 from typing import IO, List, Dict, Union, Tuple, Any, Iterable, Sequence, Mapping
+import typing as T
 from warnings import warn
 
 import yaml
@@ -17,7 +18,6 @@ from .data_generator_runtime_object_model import (
     FieldFactory,
     SimpleValue,
     StructuredValue,
-    ReferenceValue,
     Statement,
     Definition,
 )
@@ -79,6 +79,7 @@ class TableInfo:
 
 class ParseContext:
     current_parent_object = None
+    top_level = True
 
     def __init__(self):
         self.macros = {}
@@ -136,6 +137,10 @@ class ParseContext:
         self.table_infos[template.tablename] = table_info
         table_info.register(template)
 
+    @property
+    def top_level(self):
+        return self.current_parent_object is None
+
 
 def removeline_numbers(dct: Dict) -> Dict:
     return {i: dct[i] for i in dct if i != "__line__"}
@@ -185,15 +190,19 @@ def parse_structured_value(name: str, field: Dict, context: ParseContext) -> Def
         namespace, name = function_name.split(".")
         plugin = context.parser_macros_plugins.get(namespace)
     if plugin:
-        func = getattr(plugin, name)
-        rc = func(context, args)
-        return rc
-    else:
-        args = parse_structured_value_args(args, context)
-        if function_name == "reference":
-            return ReferenceValue(function_name, args, **context.line_num(field))
-        else:
-            return StructuredValue(function_name, args, **context.line_num(field))
+        try:
+            func = getattr(plugin, name)
+            rc = func(context, args)
+            return rc
+        except AttributeError as e:
+            # check if this is a regular runtime function. If so,
+            # fall-through and handle it as if we had never
+            # tried to parse it as a macro plugin
+            if not hasattr(plugin, "Functions") and not hasattr(plugin.Functions, name):
+                raise e
+
+    args = parse_structured_value_args(args, context)
+    return StructuredValue(function_name, args, **context.line_num(field))
 
 
 def parse_field_value(
@@ -291,6 +300,22 @@ def parse_inclusions(
         friends.extend(include_friends)
 
 
+# TODO: Use application object to write this instead
+def check_identifier(name: T.Optional[str], source: dict, context: str):
+    if not name:
+        return
+    badchars = set('."')
+    if set(name).intersection(badchars):
+        line_info = source["__line__"]
+        warn(
+            f"{name} is not a valid nickname.\n"
+            f"{context} cannot contain '.' or '\"'."
+            "Future versions of Snowfakery may disallow it.\n"
+            f"{line_info.filename}:{line_info.line_num}",
+            stacklevel=100,
+        )
+
+
 def parse_object_template(yaml_sobj: Dict, context: ParseContext) -> ObjectTemplate:
     parsed_template: Any = parse_element(
         dct=yaml_sobj,
@@ -306,11 +331,14 @@ def parse_object_template(yaml_sobj: Dict, context: ParseContext) -> ObjectTempl
         },
         context=context,
     )
+    if not context.top_level and parsed_template.just_once:
+        raise exc.DataGenSyntaxError("just_once can only be used at the top level")
 
     assert yaml_sobj
     with context.change_current_parent_object(yaml_sobj):
         sobj_def = {}
         sobj_def["tablename"] = parsed_template.object
+        check_identifier(parsed_template.object, yaml_sobj, "Object names")
         fields: List
         friends: List
         sobj_def["fields"] = fields = []
@@ -319,12 +347,7 @@ def parse_object_template(yaml_sobj: Dict, context: ParseContext) -> ObjectTempl
         fields.extend(parse_fields(parsed_template.fields or {}, context))
         friends.extend(parse_friends(parsed_template.friends or [], context))
         sobj_def["nickname"] = nickname = parsed_template.nickname
-        if nickname and not nickname.isidentifier():
-            warn(
-                f"{nickname} is not a valid nickname.\n"
-                "Future versions of Snowfakery may disallow it."
-            )
-
+        check_identifier(nickname, yaml_sobj, "Nicknames")
         sobj_def["just_once"] = parsed_template.just_once or False
         sobj_def["line_num"] = parsed_template.line_num.line_num
         sobj_def["filename"] = parsed_template.line_num.filename
@@ -582,7 +605,7 @@ def parse_file(stream: IO[str], context: ParseContext) -> List[Dict]:
 
     if not isinstance(data, list):
         raise exc.DataGenSyntaxError(
-            "Generator file should be a list (use '-' on top-level lines)",
+            "Recipe file should be a list (use '-' on top-level lines)",
             stream_name,
             1,
         )
