@@ -12,6 +12,7 @@ from faker.providers import BaseProvider as FakerProvider
 
 import snowfakery.data_gen_exceptions as exc
 from .utils.yaml_utils import SnowfakeryDumper
+from .utils.collections import CaseInsensitiveDict
 
 from numbers import Number
 
@@ -100,6 +101,10 @@ class PluginContext:
             self.plugin.__class__.__name__
         )
 
+    def unique_context_identifier(self) -> str:
+        "An identifier that will be unique across iterations (but not portion invocations)"
+        return self.interpreter.current_context.unique_context_identifier
+
     def evaluate_raw(self, field_definition):
         """Evaluate the contents of a field definition"""
         return field_definition.render(self.interpreter.current_context)
@@ -114,6 +119,10 @@ class PluginContext:
         else:
             raise f"Cannot simplify {field_definition}. Perhaps should have used evaluate_raw?"
 
+    @property
+    def current_filename(self):
+        return self.interpreter.current_context.current_template.filename
+
 
 def lazy(func: Any) -> Callable:
     """A lazy function is one that expects its arguments to be unparsed"""
@@ -125,6 +134,14 @@ def resolve_plugins(
     plugin_specs: List[Tuple[str, object]], search_paths: List[Union[str, Path]]
 ):
     "Resolve a list of plugins and lineinfos"
+    with plugin_path(search_paths):
+        plugins = []
+        for plugin_spec in plugin_specs:
+            plugins.extend(resolve_plugin(*plugin_spec))
+        return plugins
+
+
+def plugin_path(search_paths):
     cwd_plugins = "./plugins"
     user_plugins = Path.home() / ".snowfakery/plugins"
     new_sys_path = [
@@ -133,25 +150,29 @@ def resolve_plugins(
         str(cwd_plugins),
         str(user_plugins),
     ]
-
-    with patch.object(sys, "path", new_sys_path):
-        return [resolve_plugin(*plugin_spec) for plugin_spec in plugin_specs]
+    return patch.object(sys, "path", new_sys_path)
 
 
 def resolve_plugin(plugin: str, lineinfo) -> object:
     """Resolve a plugin to a class"""
-    cls = _resolve_plugin_alternatives(plugin)
+    cls = resolve_plugin_alternatives(plugin)
     if not cls:
         raise exc.DataGenImportError(
             f"Cannot find plugin: {plugin}", lineinfo.filename, lineinfo.line_num
         )
 
+    categories = []
+
     if issubclass(cls, FakerProvider):
-        return (FakerProvider, cls)
-    elif issubclass(cls, SnowfakeryPlugin):
-        return (SnowfakeryPlugin, cls)
-    elif issubclass(cls, ParserMacroPlugin):
-        return (ParserMacroPlugin, cls)
+        categories.append((FakerProvider, cls))
+    else:
+        if issubclass(cls, SnowfakeryPlugin):
+            categories.append((SnowfakeryPlugin, cls))
+        if issubclass(cls, ParserMacroPlugin):
+            categories.append((ParserMacroPlugin, cls))
+
+    if categories:
+        return categories
     else:
         raise exc.DataGenTypeError(
             f"{cls} is not a Faker Provider nor Snowfakery Plugin",
@@ -160,7 +181,7 @@ def resolve_plugin(plugin: str, lineinfo) -> object:
         )
 
 
-def _resolve_plugin_alternatives(plugin):
+def resolve_plugin_alternatives(plugin):
     """Interpret the plugin declaration in 2 ways.
     Return it when we find one that matches.
 
@@ -189,6 +210,7 @@ def _resolve_plugin_alternatives(plugin):
         try:
             module = import_module(module_name)
             if hasattr(module, class_name):
+                assert getattr(module, class_name)
                 return getattr(module, class_name)
         except ModuleNotFoundError:
             pass
@@ -202,7 +224,7 @@ class PluginResult:
     PluginResults are serialized to continuation files as dicts."""
 
     def __init__(self, result: Mapping):
-        self.result = result
+        self.result = CaseInsensitiveDict(result)
 
     def __getattr__(self, name):
         # ensures that it won't recurse
@@ -218,6 +240,21 @@ class PluginResult:
         return str(self.result)
 
 
+class PluginOption:
+    def __init__(self, name, typ):
+        self.name = name
+        self.type = typ
+
+    def convert(self, value):
+        try:
+            return self.type(value)
+        except TypeError as e:
+            raise TypeError(
+                f"{self.name} option is wrong type {type(value)} rather than {self.type}",
+                *e.args,
+            )
+
+
 # round-trip PluginResult objects through continuation YAML if needed.
 SnowfakeryDumper.add_representer(PluginResult, Representer.represent_object)
 yaml.SafeLoader.add_constructor(
